from __future__ import print_function

from kabaret import flow


class NoDialogAction(flow.Action):

    def needs_dialog(self):
        return False

    def run(self, button):
        print('Well done Jolly Jumper !')

class SimpleDialogAction(flow.Action):

    def needs_dialog(self):
        return True

    def get_buttons(self):
        self.message.set('This is the action message: <3')
        return ['Ok', 'Maybe', 'Nope']

    def run(self, button):
        print('You chose {}'.format(button))

class DialogParamsAction(flow.Action):

    cut_down_trees = flow.BoolParam(True)
    skip_and_jump = flow.BoolParam(True)
    press_wild_flowers = flow.BoolParam(True)

    def needs_dialog(self):
        return True

    def get_buttons(self):
        self.message.set('Do you:')
        return ['Ok']

    def run(self, button):
        all_true = (
            self.cut_down_trees.get()
            and self.skip_and_jump.get()
            and self.press_wild_flowers.get()
        )
        if (
            self.cut_down_trees.get()
            and self.skip_and_jump.get()
            and self.press_wild_flowers.get()
        ):
            print('You hang around in bars')
        else:
            print('Do you ever party ?')

class DispatchParams(flow.Object):

    pool = flow.Param('Farm')
    priority = flow.Param(50)

    def get_flags(self):
        return [
            '-P', str(self.pool.get()),
            '-p', str(self.priority.get()),
        ]


class SequenceParam(flow.Object):

    first_frame = flow.Param(1)
    last_frame = flow.Param(100)

    def get_flags(self):
        return [
            '--first', str(self.first_frame.get()),
            '--last', str(self.last_frame.get()),
        ]

class ComplexDialogAction(flow.Action):

    dispatch = flow.Child(DispatchParams)
    sequence = flow.Child(SequenceParam)

    def needs_dialog(self):
        return True

    def get_buttons(self):
        self.message.set('Configure and Submit your job')
        return ['Submit']

    def run(self, button):
        cmd = ['spam_it']+self.dispatch.get_flags()+self.sequence.get_flags()
        print('#---> Cmd:', ' '.join(cmd))

class EditMyValueAction(flow.Action):

    _value = flow.Parent()

    def needs_dialog(self):
        return True

    def get_buttons(self):
        self.message.set('Pick a color:')
        return ['Red', 'Pink', 'Blue', 'Violet']

    def run(self, button):
        self._value.set(button)

class MyValue(flow.values.Value):

    edit = flow.Child(EditMyValueAction)

class SubDialogAction(flow.Action):

    quote = flow.Param('', MyValue)

    def needs_dialog(self):
        return True


    def run(self, button):
        print('You selected: "{}"'.format(self.quote.get()))

class ActionsGroup(flow.Object):

    no_dialog_action = flow.Child(NoDialogAction)
    simple_dialog_action = flow.Child(SimpleDialogAction)
    dialog_params_action = flow.Child(DialogParamsAction)
    complex_dialog_action = flow.Child(ComplexDialogAction)

    sub_dialog_action = flow.Child(SubDialogAction)


'''

    kabaret.flow.action

    Defines the Action class, an object on which UI commands can call
    the run() method to execute some code.

    The return value of run() dictates some UI behavior
    (see Action.run() doc).

    Defines the DropAction class, an object used to handle GUI drops
    on the ConnectAction parent.
    (NB: it is not a subclasse of Action and wont show up as a button)

'''

from .object import Object
from .relations import SessionParam, Parent


class Action(Object):

    ICON = 'action'

    # does this action shows in parent's inline GUI ?
    SHOW_IN_PARENT_INLINE = True
    # does this action shows in parent's detailed GUI ?
    SHOW_IN_PARENT_DETAILS = True

    message = SessionParam().ui(editor='label', wrap=True).ui(editable=False, label='')

    @classmethod
    def get_result(
            cls, close=None, refresh=None,
            goto=None, next_action=None
    ):
        '''
        The value returned by run(button) will be inspected by callers to
        decide how they should react.
        This helper class method will generate a result matching its args:
            close: (bool)
                Prevent the action dialog to be closed when set to False
            refresh: (bool)
                Force a reload of the parent page avec dialog close
                (This should not be needed anymore thanks to touch() calls...)
            goto: (oid)
                Force the caller to load the given oid page.
            next_action: (oid)
                Force the dialog to not close but load the ui for the action
                with the given oid.
                This is the way to implement "Wizard Pages"

        '''
        ret = {}
        if close is not None:
            ret['close'] = close and True or False
        if refresh is not None:
            ret['refresh'] = refresh and True or False
        if goto is not None:
            ret['goto'] = goto

        if next_action is not None:
            ret['next_action'] = next_action

        return ret

    def needs_dialog(self):
        '''
        May be overriden by subclasses to return False if the action
        does not need to show a dialog.
        Action without dialog are called with run(button=None), plus the 'goto'
        and 'next_action' fields of the returned value do the same.

        NB: If the action cannot run, it should show a dialog wiht a
        desciption why in self.message
        '''
        return True

    def get_buttons(self):
        '''
        Returns a list of string suitable as the 'button' argument for a
        call to self.run().

        NB: If the action cannot run, you should set a desciption why
        in self.message, return ['Cancel'] and handle that in run().
        This is far better than returning False from needs_dialog() or let
        the run() method decide to do nothing since the user will not have
        a clear feedback showing that nothing happened...
        '''
        return ['Run']

    def run(self, button):
        '''
        The return value should be None or the return value of
        a self.get_result() call.
        GUI will inspect the returned value and act upon it.
        '''
        self.message.set('This is an abstract action and it cannot be used.')
        return self.get_result(close=False)


class ConnectAction(Object):
    '''
    Subclasses will want to overwrite:
        - accept_label: return the label to show in GUI if the objects and urls are
        acceptable, None otherwise.
        - run: to do the job.

    The run method is guaranted to be called only if accept_label did not return None.
    '''

    def accept_label(self, objects, urls):
        return 'Drop %i Oject(s)/File(s) here' % (len(objects) + len(urls),)

    def run(self, objects, urls):
        raise NotImplementedError()


class ChoiceValueSetAction(Action):

    SHOW_IN_PARENT_INLINE = False
    VALUE_TO_SET = None

    _choice_value = Parent()

    def needs_dialog(self):
        return False

    def run(self, button):
        self._choice_value.set(self.VALUE_TO_SET)


class ChoiceValueSelectAction(Action):

    SHOW_IN_PARENT_INLINE = False

    _choice_value = Parent()

    def get_buttons(self):
        return self._choice_value.choices()

    def run(self, button):
        self._choice_value.set(button)




import six

from qtpy import QtWidgets, QtGui, QtCore


QWidgetType = type(QtWidgets.QWidget)


class ViewType(QWidgetType):

    def __new__(cls, class_name, bases, class_dict):
        class_dict['_VIEWS'] = []
        return super(ViewType, cls).__new__(cls, class_name, bases, class_dict)


# six.with_metaclass does not play well with PySide's shiboken.
if six.PY2:
    class _BaseView(QtWidgets.QWidget):
        __metaclass__ = ViewType

else:
    class _BaseView(six.with_metaclass(ViewType, QtWidgets.QWidget)):
        pass


class View(_BaseView):

    _VIEWS = []  # will be reset to [] for all view types by the metaclass

    @classmethod
    def view_type_name(cls):
        return cls.__name__

    def __init__(self, session, main_window_manager, parent):
        super(View, self).__init__(parent)

        self.__class__._VIEWS.append(self)

        self.session = session
        self._main_window_manager = main_window_manager

        self._view_title = self.view_type_name()
        self._on_view_title_change = None

        # Menu
        self.menubar = QtWidgets.QMenuBar(self)
        self.view_menu = self.menubar.addMenu(self.view_title())

        # Tools
        self._header_tools = {}
        self._header_tools_layout = QtWidgets.QHBoxLayout()

        content_widget = QtWidgets.QWidget(self)

        lo = QtWidgets.QVBoxLayout()
        lo.setContentsMargins(0, 0, 0, 0)
        lo.setSpacing(0)
        self.setLayout(lo)

        hlo = QtWidgets.QHBoxLayout()
        hlo.setContentsMargins(0, 0, 0, 0)
        hlo.addWidget(self.menubar)
        header_widgets_layout = QtWidgets.QHBoxLayout()
        hlo.addStretch()
        hlo.addLayout(header_widgets_layout, 100)
        hlo.addLayout(self._header_tools_layout)
        lo.addLayout(hlo)
        top_layout = QtWidgets.QHBoxLayout()
        lo.addLayout(top_layout)
        lo.addWidget(content_widget, 100)
        self._build(
            self, top_layout, content_widget,
            self, header_widgets_layout
        )

        self._update_menus()

    def _create_scrolled_area(self, parent):
        scroll_area = QtWidgets.QScrollArea(parent)
        scroll_area.setWidgetResizable(True)

        content_widget = QtWidgets.QWidget(parent)
        scroll_area.setWidget(content_widget)

        return scroll_area, content_widget

    def _build(
            self,
            top_parent, top_layout,
            main_parent,
            header_parent, header_layout
    ):
        '''
        Subclasses must override this to build their widgets.
        '''
        pass

    def duplicate_view(self, *extra_args, **extra_kwargs):
        dock = self.dock_widget()

        area = dock.dock_area()
        area = {
            QtCore.Qt.LeftDockWidgetArea: QtCore.Qt.RightDockWidgetArea,
            QtCore.Qt.RightDockWidgetArea: QtCore.Qt.BottomDockWidgetArea,
            QtCore.Qt.BottomDockWidgetArea: QtCore.Qt.LeftDockWidgetArea,
            QtCore.Qt.TopDockWidgetArea: QtCore.Qt.LeftDockWidgetArea,
        }[area]

        return self._main_window_manager.add_view(
            self.view_type_name(),
            area=area,
            *extra_args, **extra_kwargs
        )

    def set_on_view_title_change(self, f):
        self._on_view_title_change = f

    def set_view_title(self, title):
        self._view_title = title
        if self._on_view_title_change is not None:
            self._on_view_title_change()

    def view_title(self):
        return self._view_title

    def add_header_tool(self, tool_id, label, tooltip, f):
        def pyside_and_pyqt_handler(checked=None, f=f):
            f()
        b = QtWidgets.QToolButton(self)
        b.setProperty('no_border', True)
        self._header_tools[tool_id] = b
        b.setText(label)
        b.clicked.connect(pyside_and_pyqt_handler)
        b.setToolTip(tooltip)
        self._header_tools_layout.addWidget(b)
        return b

    def remove_header_tool(self, tool_id):
        self._header_tools[tool_id].deleteLater()

    def _update_menus(self):
        pass

    def dock_widget(self):
        return self.parentWidget()

    def delete_on_close(self):
        '''
        If returns True, the view will be deleted when its dock is closed.

        Default is to return True unless this is the last instance of this view type.
        '''
        try:
            self.__class__._VIEWS[1]
        except IndexError:
            return False
        return True

    def about_to_delete(self):
        '''
        Called if delete_on_close() returned True and the view is about to be deleted.
        Subclasses may override this to do some cleanup or cancel some pending commands but
        MUST call the base implementation
        '''
        self.__class__._VIEWS.remove(self)

    def on_hide(self):
        '''
        Called when the view's dock is hidden.
        Beware that consecutive hide/show happens when floating a view...
        '''
        pass

    def on_show(self):
        '''
        Called when the view's dock is shown.
        Beware that consecutive hide/show happens when floating a view...
        '''
        pass

    def receive_event(self, event, data):
        raise NotImplementedError()
